@@ -51,6 +51,7 @@ public class JavaBridge {
  static java.lang.Object oldEventSource = null;
  static String oldEventName = null;
  static int oldDetail1 = -1;
+ static boolean debugFlag = false;
 
  javax.accessibility.Accessible appAccessible;
 
@@ -117,14 +118,6 @@ public class JavaBridge {
 					    }
 					}
 				}
-//				else if (source instanceof AccessibleContext) 
-//					System.out.println ("Selection source role: " +
-//                                         ((AccessibleContext) source).getAccessibleRole());
-//				else if (source instanceof javax.accessibility.Accessible)
-//					System.out.println ("Selection role " +
-//				          ((javax.accessibility.Accessible) source).
-//							    getAccessibleContext().
-//							    getAccessibleRole());
 				if ( textEvent ) {
 				    int selection_start = 0, selection_end = 0;
 				    int selection_change_start = 0, selection_change_end = 0;
@@ -332,7 +325,12 @@ public class JavaBridge {
 	}
 
 	public JavaBridge() {
-		System.err.println ("Java Accessibility Bridge for GNOME loaded.\n");
+
+		setDebug();
+
+		if (debugFlag)
+			System.err.println ("Java Accessibility Bridge for GNOME loaded.\n");
+
 		// Not sure what kind of arguments should be sent to ORB
 		String vm_rev = System.getProperty("java.version");
 
@@ -425,7 +423,8 @@ public class JavaBridge {
 			accessible.ref();
 			AccessUtil.getRegistryObject().notifyEvent(event);
 		} catch (Exception ex) {
-			System.out.println ("Error dispatching event "+eventName+":"+eventSource+"; "+ex);
+			if (debugFlag)
+				System.out.println ("Error dispatching event "+eventName+":"+eventSource+"; "+ex);
 		}		
 	}
 
@@ -446,7 +445,8 @@ public class JavaBridge {
 		}
 		catch (Exception e)
 		{
-			System.out.println("Exception in dispatchDeviceEvent " + e);
+			if (debugFlag)
+				System.out.println("Exception in dispatchDeviceEvent " + e);
 			return false;
 		}
 	}
@@ -552,7 +552,8 @@ public class JavaBridge {
 			accessible.ref();
 			AccessUtil.getRegistryObject().notifyEvent(event);
 		} catch (Exception ex) {
-			System.out.println ("Error dispatching focus event "+":"+eventSource+":"+ex);
+			if (debugFlag)
+				System.out.println ("Error dispatching focus event "+":"+eventSource+":"+ex);
 		}		
 	}
 
@@ -964,7 +965,8 @@ public class JavaBridge {
             }
             catch ( Exception e ) {
                 // System.err.println("AttachListener: exception caught " + e.toString() );
-                e.printStackTrace();
+		if (debugFlag) 
+			e.printStackTrace();
             }
         }
     }
@@ -1053,23 +1055,31 @@ public class JavaBridge {
 				return false;
 			}
 
-		System.err.println ("Registering Application.");
-            appAccessible = new ApplicationAccessible ();
+			if (AccessUtil.getRegistryObject() == null)
+				return false;
+
+			if (debugFlag)
+				System.err.println ("Registering Application.");
+			appAccessible = new ApplicationAccessible ();
             
-            Application aref = getAccessibleObjectFactory().
+			Application aref = getAccessibleObjectFactory().
                                getApplication(appAccessible.getAccessibleContext());
             
-		    System.err.println(aref);
+			if (debugFlag)
+				System.err.println(aref);
             
-		    AccessUtil.getRegistryObject().registerApplication(aref);
+			AccessUtil.getRegistryObject().registerApplication(aref);
 		    
-		    System.err.println(aref.name() + ":" + aref.id());
-		    System.err.println("Just registered Application");
-		    return true;
+			if (debugFlag) {
+				System.err.println(aref.name() + ":" + aref.id());
+				System.err.println("Just registered Application");
+			}
+			return true;
 			
 		} catch (Exception e) {
-                    System.out.println("Error registering application, exception is: "+ e);
-                    return false;
+			if (debugFlag)
+				System.out.println("Error registering application, exception is: "+ e);
+			return false;
 		}
 	}
 	
@@ -1144,6 +1154,8 @@ public class JavaBridge {
 				isConsumed = JavaBridge.dispatchDeviceEvent (de);
 				if ( isConsumed ) pendingKeyPress = null;
 			} else if (e.getID () == KeyEvent.KEY_RELEASED) {
+				if (pendingKeyPress != null) 
+					super.dispatchEvent (pendingKeyPress);
 				de = KeyEventAdapter.deviceEventFromKeyEvent (e);
 				isConsumed = JavaBridge.dispatchDeviceEvent (de);
 				pendingKeyPress = null;
@@ -1162,4 +1174,25 @@ public class JavaBridge {
 	  
   }
 
+	private void setDebug() {
+		try {
+			Process p = Runtime.getRuntime().exec ("env");
+			BufferedReader b = new BufferedReader (
+				new InputStreamReader (p.getInputStream ()));
+
+			String result = b.readLine();
+			if (result == null) 
+				return;
+
+			String str;
+			while ((str = b.readLine()) != null) {
+				if (str.indexOf ("GNOME_ACCESSIBILITY") >= 0) {
+					debugFlag = true;
+					return;
+				}
+			}
+		}
+		catch (Exception e) {
+		}
+	}
 }
